"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.writeClosureToDir = void 0;
const fs_extra_1 = require("fs-extra");
const compile_1 = require("./compile");
function writeClosureToDir(imports, locations, dependentResources, dirpath) {
    const sourceCode = generateSourceCode(imports, locations, dependentResources);
    (0, fs_extra_1.ensureDirSync)(dirpath);
    const filepath = dirpath + "/index.ts";
    (0, fs_extra_1.writeFileSync)(filepath, sourceCode);
    (0, compile_1.compile)(filepath, dirpath, /* bundle */ true);
}
exports.writeClosureToDir = writeClosureToDir;
function generateSourceCode(imports, locations, dependentResources) {
    let cirCode = imports + "\n";
    const resources = new Set();
    // Find the dependencies of this CIR and build corresponding instances.
    for (const dependentRes of dependentResources) {
        if (resources.has(dependentRes.name)) {
            continue;
        }
        resources.add(dependentRes.name);
        // TODO: verify if the buildClient function exists. If it does not, use the original statement.
        cirCode += dependentRes.imports + "\n";
        cirCode += `const ${dependentRes.name} = ${dependentRes.type}.buildClient(${dependentRes.parameters});\n`;
    }
    const fileSelections = {};
    locations.forEach((loc) => {
        if (!(loc.file in fileSelections)) {
            fileSelections[loc.file] = [];
        }
        const startPos = loc.linenum["start"].split("-").map((n) => Number(n));
        const endPos = loc.linenum["end"].split("-").map((n) => Number(n));
        fileSelections[loc.file].push({
            depth: loc.depth,
            start: startPos,
            end: endPos,
        });
    });
    if (Object.keys(fileSelections).length != 1) {
        throw new Error(`Currently, Pluto can only support single file.`);
    }
    const fileCodes = []; // file, code
    for (const file in fileSelections) {
        const curFileCode = genFileCode(file, fileSelections[file]);
        fileCodes.push([file, curFileCode]);
    }
    return cirCode + fileCodes[0][1];
}
function genFileCode(file, segments) {
    segments.sort((a, b) => {
        if (a.start[0] != b.start[0])
            return a.start[0] - b.start[0];
        return a.start[1] - b.start[1];
    });
    const usercode = (0, fs_extra_1.readFileSync)(file, "utf-8");
    const lines = usercode.split("\n");
    let curFileCode = "";
    for (const segment of segments) {
        const [startLine, startPos] = segment.start;
        const [endLine, endPos] = segment.end;
        let curSegCode = "";
        // Iterate through the range of this segment and construct the code.
        for (let lineIdx = startLine; lineIdx <= endLine; lineIdx++) {
            const linecode = lines[lineIdx];
            let curLineCode = "";
            if (lineIdx == startLine) {
                if (segment.depth == 0)
                    curLineCode = `export default `;
                curLineCode += linecode.slice(startPos);
            }
            else if (lineIdx == endLine) {
                curLineCode = linecode.slice(0, endPos);
            }
            else {
                curLineCode = linecode;
            }
            curSegCode += curLineCode + "\n";
        }
        curFileCode += curSegCode + "\n";
    }
    return curFileCode;
}
//# sourceMappingURL=store-closure.js.map