import ts from "typescript";
import { arch } from "@plutolang/base";
import { ImportElement } from "./imports";
export interface VisitResult {
    resourceRelatInfos?: ResourceRelationshipInfo[];
    resourceVarInfos?: ResourceVariableInfo[];
}
export declare function concatVisitResult(...visitResults: (VisitResult | undefined)[]): VisitResult;
export interface ResourceVariableInfo {
    varName: string;
    resourceName?: string;
    resourceConstructInfo: ResourceConstructInfo;
}
export interface ResourceRelationshipInfo {
    fromVarName: string;
    toVarNames: string[];
    type: arch.RelatType;
    operation: string;
    parameters: ParameterInfo[];
}
export interface ResourceConstructInfo {
    constructExpression: string;
    importElements: ImportElement[];
    parameters?: ParameterInfo[];
    locations: Location[];
}
export type ParameterInfo = TextParameterInfo | ClosureParameterInfo | PropertyParameterInfo;
export interface BaseParameterInfo {
    type: "text" | "closure" | "property";
    name: string;
    order: number;
    expression?: ts.Expression;
}
interface TextParameterInfo extends BaseParameterInfo {
    type: "text";
}
interface ClosureParameterInfo extends BaseParameterInfo {
    type: "closure";
    closureName: string;
}
interface PropertyParameterInfo extends BaseParameterInfo {
    type: "property";
    resourceVarName: string;
    property: string;
}
export interface Location {
    file: string;
    depth: number;
    start: string;
    end: string;
}
export {};
